@@ -1,5 +1,5 @@
 /* tpb - program to use the IBM ThinkPad(tm) special keys
- * Copyright (C) 2002,2003 Markus Braun <markus.braun@krawel.de>
+ * Copyright (C) 2002-2004 Markus Braun <markus.braun@krawel.de>
  *
  * This file is part of tpb.
  *
@@ -42,7 +42,7 @@
 
 /* RCS version string for later identification using rcs ident {{{*/
 #ifndef lint
-static volatile const char *RCSid = "@(#) $Id: cfg.c,v 1.11 2003/12/26 20:00:35 uid66905 Exp $";
+static volatile const char *RCSid = "@(#) $Id: cfg.c,v 1.13 2004/04/29 11:12:57 mbr Exp $";
 #endif /* lint */
 /* }}} */
 
@@ -193,15 +193,16 @@ void parse_option(config *cfg, int argc, char **argv) { /* {{{ */
       {"home",     1, NULL, 'H'},
       {"search",   1, NULL, 'S'},
       {"mail",     1, NULL, 'M'},
+      {"wireless", 1, NULL, 'W'},
       {"xevents",  1, NULL, 'x'},
       {"verbose",  0, NULL, 'v'},
       {NULL,       0, NULL,   0}
     };
 
 #ifdef HAVE_LIBXOSD
-    next_option = getopt_long (argc, argv, "hdA:P:c:C:m:o:p:t:H:S:M:x:v", long_options, &option_index);
+    next_option = getopt_long (argc, argv, "hdA:P:c:C:m:o:p:t:H:S:M:W:x:v", long_options, &option_index);
 #else /* HAVE_LIBXOSD */
-    next_option = getopt_long (argc, argv, "hdA:P:c:C:m:p:t:H:S:M:x:v", long_options, &option_index);
+    next_option = getopt_long (argc, argv, "hdA:P:c:C:m:p:t:H:S:M:W:x:v", long_options, &option_index);
 #endif /* HAVE_LIBXOSD */
 
     if (next_option == -1) {
@@ -267,6 +268,10 @@ void parse_option(config *cfg, int argc, char **argv) { /* {{{ */
 	set_value(CFG_MAIL, optarg, &cmd_cfg);
 	break;
 
+      case 'W':
+	set_value(CFG_WIRELESS, optarg, &cmd_cfg);
+	break;
+
       case 'x':
 	set_value(CFG_XEVENTS, optarg, &cmd_cfg);
 	break;
@@ -341,6 +346,12 @@ void init_cfg(config *cfg) { /* {{{ */
   }
   cfg->mail_cmd = NULL;
 
+  /* cfg->wireless_cmd */
+  if(cfg->wireless_cmd != NULL) {
+    free(cfg->wireless_cmd);
+  }
+  cfg->wireless_cmd = NULL;
+
   /* cfg->favorites_cmd */
   if(cfg->favorites_cmd != NULL) {
     free(cfg->favorites_cmd);
@@ -490,6 +501,9 @@ void clear_cfg(config *cfg) { /* {{{ */
   /* cfg->mail_cmd */
   cfg->mail_cmd = NULL;
 
+  /* cfg->wireless_cmd */
+  cfg->wireless_cmd = NULL;
+
   /* cfg->favorites_cmd */
   cfg->favorites_cmd = NULL;
 
@@ -595,6 +609,11 @@ void free_cfg(config *cfg) { /* {{{ */
     free(cfg->mail_cmd);
   }
 
+  /* cfg->wireless_cmd */
+  if(cfg->wireless_cmd != NULL) {
+    free(cfg->wireless_cmd);
+  }
+
   /* cfg->favorites_cmd */
   if(cfg->favorites_cmd != NULL) {
     free(cfg->favorites_cmd);
@@ -719,6 +738,14 @@ void override_cfg(config *master, config *slave) { /* {{{ */
     slave->mail_cmd = strdup(master->mail_cmd);
   }
 
+  /* cfg->wireless_cmd */
+  if(master->wireless_cmd != NULL) {
+    if(slave->wireless_cmd != NULL) {
+      free(slave->wireless_cmd);
+    }
+    slave->wireless_cmd = strdup(master->wireless_cmd);
+  }
+
   /* cfg->favorites_cmd */
   if(master->favorites_cmd != NULL) {
     if(slave->favorites_cmd != NULL) {
@@ -1000,6 +1027,16 @@ void set_value(char *key, char *arg, config *cfg) { /* {{{ */
     }
   } /* }}} */
 
+  else if (strcmp(CFG_WIRELESS, key) == 0) { /* {{{ */
+    if(cfg->wireless_cmd != NULL) {
+      free(cfg->wireless_cmd);
+    }
+    if((cfg->wireless_cmd = strdup(arg)) == NULL) {
+      fputs(_("Not enough memory"), stderr);
+      _exit(1);
+    }
+  } /* }}} */
+
   else if (strcmp(CFG_FAVORITES, key) == 0) { /* {{{ */
     if(cfg->favorites_cmd != NULL) {
       free(cfg->favorites_cmd);
@@ -1312,7 +1349,7 @@ void find_nvram(config *cfg) /* {{{ */
 /* print the usage to sdtout */
 void print_usage(const char *prog) /* {{{ */
 {
-  printf("%s " VERSION " (C) 2002-2003 Markus Braun \n\n", prog);
+  printf("%s " VERSION " (C) 2002-2004 Markus Braun \n\n", prog);
   printf(_("Usage: %s options\n\n"), prog);
   printf(_("options:\n"));
   printf(_("   -h, --help           display this help\n"));
@@ -1328,6 +1365,7 @@ void print_usage(const char *prog) /* {{{ */
   printf(_("   -H, --home=CMD       command and options for Home button [none]\n"));
   printf(_("   -S, --search=CMD     command and options for Search button [none]\n"));
   printf(_("   -M, --mail=CMD       command and options for Mail button [none]\n"));
+  printf(_("   -W, --wireless=CMD   command and options for Wireless button [none]\n"));
   printf(_("   -C, --callback=CMD   string with command that should be executed for each\n"));
   printf(_("                        pressed button. It is called with pressed button as\n"));
   printf(_("                        first argument and new state as second [none]\n"));
