@@ -1,5 +1,5 @@
 /* tpb - program to use the IBM ThinkPad(tm) special keys
- * Copyright (C) 2002,2003 Markus Braun <markus.braun@krawel.de>
+ * Copyright (C) 2002-2004 Markus Braun <markus.braun@krawel.de>
  *
  * This file is part of tpb.
  *
@@ -46,7 +46,11 @@ enum mode {MODE_COMMENT, MODE_INDENT, MODE_KEYWORD, MODE_SEPARATOR, MODE_ARGUMEN
 #ifdef HAVE_LIBXOSD
 #define DEFAULT_OSD STATE_ON
 #define DEFAULT_OSDLINES 2
+#if HAVE_LIBXOSD_VERSION > 700
+#define DEFAULT_OSDFONT osd_default_font
+#else
 #define DEFAULT_OSDFONT "10x20"
+#endif /* HAVE_LIBXOSD_VERSION */
 #define DEFAULT_OSDCOLOR "BLUE"
 #define DEFAULT_OSDTIMEOUT 3
 #define DEFAULT_OSDSHADOW 2
@@ -84,6 +88,7 @@ enum mode {MODE_COMMENT, MODE_INDENT, MODE_KEYWORD, MODE_SEPARATOR, MODE_ARGUMEN
 #define CFG_HOME "home"
 #define CFG_SEARCH "search"
 #define CFG_MAIL "mail"
+#define CFG_WIRELESS "wireless"
 #define CFG_FAVORITES "favorites"
 #define CFG_RELOAD "reload"
 #define CFG_ABORT "abort"
@@ -135,6 +140,7 @@ typedef struct {
   char *home_cmd;
   char *search_cmd;
   char *mail_cmd;
+  char *wireless_cmd;
   char *favorites_cmd;
   char *reload_cmd;
   char *abort_cmd;
