@@ -1,5 +1,5 @@
 /* tpb - program to use the IBM ThinkPad(tm) special keys
- * Copyright (C) 2002,2003 Markus Braun <markus.braun@krawel.de>
+ * Copyright (C) 2002-2004 Markus Braun <markus.braun@krawel.de>
  *
  * This file is part of tpb.
  *
@@ -22,15 +22,16 @@
 #include <ctype.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
-#include <linux/soundcard.h>
+#include <sys/wait.h>
+#include <locale.h>
+#include <signal.h>
+#include <sys/soundcard.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <wait.h>
-#include <X11/Xlib.h>
 #include "config.h"
 
 #if ENABLE_NLS
@@ -41,13 +42,17 @@
 #include <xosd.h>
 #endif /* HAVE_LIBXOSD */
 
+#ifdef HAVE_LIBX11
+#include <X11/Xlib.h>
+#endif /* HAVE_LIBX11 */
+
 #include "cfg.h"
 #include "tpb.h"
 /* }}} */
 
 /* RCS version string for later identification using rcs ident {{{ */
 #ifndef lint
-static volatile const char *RCSid = "@(#) $Id: tpb.c,v 1.15 2003/12/26 20:00:35 uid66905 Exp $";
+static volatile const char *RCSid = "@(#) $Id: tpb.c,v 1.23 2004/05/18 11:21:18 mbr Exp $";
 #endif /* lint */
 /* }}} */
 
@@ -63,15 +68,22 @@ void daemonize(void);
 #ifdef HAVE_LIBXOSD
 xosd *init_xosd(void);
 #endif /* HAVE_LIBXOSD */
+#ifdef HAVE_LIBX11
 Display *init_xgrabkey(void);
+#endif /* HAVE_LIBX11 */
 int get_nvram_state(t_thinkpad_state *thinkpad_state);
 int get_apm_state(t_thinkpad_state *thinkpad_state);
+#ifdef HAVE_LIBX11
 int xgrabkey(t_thinkpad_state *thinkpad_state, Display *display);
+#endif /* HAVE_LIBX11 */
 int fork_app(char * cmd);
 void set_nvram_volume_level(t_thinkpad_state *thinkpad_state);
 unsigned int change_volume(int change);
 int apmiser_running(void);
+#ifdef HAVE_LIBX11
 int *xerrorhandler(Display *display, XErrorEvent *event);
+#endif /* HAVE_LIBX11 */
+void sig_chld_handler(int signo);
 /* }}} */
 
 int main(int argc, char **argv) /* {{{ */
@@ -87,20 +99,23 @@ int main(int argc, char **argv) /* {{{ */
 #ifdef HAVE_LIBXOSD
   xosd *osd_ptr = NULL;
 #endif /* HAVE_LIBXOSD */
+#ifdef HAVE_LIBX11
   Display *display = NULL;
+#endif /* HAVE_LIBX11 */
 
   /* zero thinkpad_state */
   memset(&thinkpad_state, 0, sizeof(thinkpad_state));
 
-  /* ignore SIG_CHLD */
-  signal_action.sa_handler = SIG_IGN;
-#ifdef SA_NOCLDWAIT
-  signal_action.sa_flags = SA_NOCLDWAIT;
-#else /* SA_NOCLDWAIT */
-  signal_action.sa_flags = 0;
-#endif /* SA_NOCLDWAIT */
+  /* register my own handler for SIG_CHLD */
+  signal_action.sa_handler = sig_chld_handler;
+  signal_action.sa_flags = SA_NOCLDSTOP;
   sigemptyset(&signal_action.sa_mask);
-  sigaction(SIGCHLD, &signal_action, NULL);
+  if (sigaction(SIGCHLD, &signal_action, NULL)  < 0)
+  {
+    fprintf(stderr, _("Unable to register signal handler:"));
+    perror(NULL);
+    _exit(-1);
+  }
 
   /* initialize i18n */
 #ifdef ENABLE_NLS
@@ -138,10 +153,12 @@ int main(int argc, char **argv) /* {{{ */
   }
 #endif /* HAVE_LIBXOSD */
 
+#ifdef HAVE_LIBX11
   /* initialize key grabbing */
   if(cfg.xevents == STATE_ON) {
     display = init_xgrabkey();
   }
+#endif /* HAVE_LIBX11 */
 
   /* to initialize struct */
   memset(&last_thinkpad_state, 0x00, sizeof(t_thinkpad_state));
@@ -151,9 +168,11 @@ int main(int argc, char **argv) /* {{{ */
   if(cfg.apm == STATE_ON) {
     get_apm_state(&thinkpad_state);
   }
+#ifdef HAVE_LIBX11
   if(cfg.xevents == STATE_ON) {
     xgrabkey(&thinkpad_state, display);
   }
+#endif /* HAVE_LIBX11 */
 
 
   /* initialize volume */
@@ -174,9 +193,11 @@ int main(int argc, char **argv) /* {{{ */
     if(cfg.apm == STATE_ON) {
       get_apm_state(&thinkpad_state);
     }
+#ifdef HAVE_LIBX11
     if(cfg.xevents == STATE_ON) {
       xgrabkey(&thinkpad_state, display);
     }
+#endif /* HAVE_LIBX11 */
 
 
     /* determine the state of the Thinkpad button  {{{ */
@@ -274,6 +295,25 @@ int main(int argc, char **argv) /* {{{ */
       }
     } /* }}} */
 
+    /* determine the state of the wireless button  {{{ */
+    if(thinkpad_state.wireless_toggle != last_thinkpad_state.wireless_toggle &&
+       thinkpad_state.hibernate_toggle == last_thinkpad_state.hibernate_toggle) {
+      if(cfg.verbose == STATE_ON) {
+	puts(_("Wireless button pressed"));
+      }
+      if(cfg.wireless_cmd != NULL) {
+	if(fork_app(cfg.wireless_cmd) != 0) {
+	  _exit(0);
+	}
+      }
+      if(cfg.callback != NULL) {
+	snprintf(callback_cmd, sizeof(callback_cmd), "%s wireless pressed", cfg.callback);
+	if(fork_app(callback_cmd) != 0) {
+	  _exit(0);
+	}
+      }
+    } /* }}} */
+
     /* determine the state of the favorites button  {{{ */
     if(thinkpad_state.favorites_toggle != last_thinkpad_state.favorites_toggle) {
       if(cfg.verbose == STATE_ON) {
@@ -402,7 +442,8 @@ int main(int argc, char **argv) /* {{{ */
     } /* }}} */
 
     /* determine the state of display  {{{ */
-    if(thinkpad_state.display_toggle != last_thinkpad_state.display_toggle &&
+    if((thinkpad_state.display_toggle != last_thinkpad_state.display_toggle ||
+	thinkpad_state.display_state != last_thinkpad_state.display_state) &&
        thinkpad_state.hibernate_toggle == last_thinkpad_state.hibernate_toggle) {
 
       /* Some thinkpads have no hardware support to switch lcd/crt. They also
@@ -410,7 +451,8 @@ int main(int argc, char **argv) /* {{{ */
        * thinkpad_state.display_toggle changes, but thinkpad_state.display_state does
        * not change, simulate the display state.
        */
-      if(thinkpad_state.display_state == last_thinkpad_state.display_state) {
+      if(thinkpad_state.display_toggle != last_thinkpad_state.display_toggle &&
+	 thinkpad_state.display_state == last_thinkpad_state.display_state) {
 	display_state = display_state % 3 + 1;
       }
       else {
@@ -692,7 +734,7 @@ void daemonize(void) /* {{{ */
     case 0:
       break;
     case -1:
-      fprintf(stderr, _("Unable to fork daemon"));
+      fprintf(stderr, _("Unable to fork daemon:"));
       perror(NULL);
       _exit(-1);
     default:
@@ -709,7 +751,7 @@ void daemonize(void) /* {{{ */
     case 0:
       break;
     case -1:
-      fprintf(stderr, _("Unable to fork daemon"));
+      fprintf(stderr, _("Unable to fork daemon:"));
       perror(NULL);
       _exit(-1);
     default:
@@ -877,6 +919,7 @@ xosd *init_xosd(void) /* {{{ */
 } /* }}} */
 #endif /* HAVE_LIBXOSD */
 
+#ifdef HAVE_LIBX11
 /* initialize X for grabbing key events */
 Display *init_xgrabkey(void) /* {{{ */
 {
@@ -919,6 +962,7 @@ Display *init_xgrabkey(void) /* {{{ */
 
   return(display);
 } /* }}} */
+#endif /* HAVE_LIBX11 */
 
 /* get the current state from the nvram */
 int get_nvram_state(t_thinkpad_state *thinkpad_state) /* {{{ */
@@ -954,6 +998,7 @@ int get_nvram_state(t_thinkpad_state *thinkpad_state) /* {{{ */
   thinkpad_state->home_toggle       = (thinkpad_state->home_toggle       & ~0x01) | (( buffer[0x56] & 0x01)     );
   thinkpad_state->search_toggle     = (thinkpad_state->search_toggle     & ~0x01) | (( buffer[0x56] & 0x02) >> 1);
   thinkpad_state->mail_toggle       = (thinkpad_state->mail_toggle       & ~0x01) | (( buffer[0x56] & 0x04) >> 2);
+  thinkpad_state->wireless_toggle   = (thinkpad_state->wireless_toggle   & ~0x01) | (( buffer[0x56] & 0x20) >> 5);
   thinkpad_state->thinklight_toggle = (thinkpad_state->thinklight_toggle & ~0x01) | (( buffer[0x58] & 0x10) >> 4);
   thinkpad_state->hibernate_toggle  = (thinkpad_state->hibernate_toggle  & ~0x01) | (( buffer[0x58] & 0x01)     );
   thinkpad_state->display_state     =                                               (( buffer[0x59] & 0x03)     );
@@ -1049,6 +1094,7 @@ int get_apm_state(t_thinkpad_state *thinkpad_state) /* {{{ */
   return 0;
 } /* }}} */
 
+#ifdef HAVE_LIBX11
 /* get key events from X */
 int xgrabkey(t_thinkpad_state *thinkpad_state, Display *display) /* {{{ */
 {
@@ -1103,15 +1149,20 @@ int xgrabkey(t_thinkpad_state *thinkpad_state, Display *display) /* {{{ */
 
   return 0;
 } /* }}} */
+#endif /* HAVE_LIBX11 */
 
 /* fork an application */
 int fork_app(char * cmd) /* {{{ */
 {
-  /* char *args[4];  */
-  /* args[0] = "sh"; */
-  /* args[1] = "-c"; */
-  /* args[2] = cmd;  */
-  /* args[3] = 0;    */
+  enum mode_t {SKIP, IN_QUOTED, IN_UNQUOTED};
+
+  char *cmd_cpy;
+  int cmd_len;
+  char *args[CALLBACK_CMD_ARGS]; 
+  int ptr;
+  int argc;
+  enum mode_t mode;
+  char quote_char;
 
   switch(fork()) {
     case -1:
@@ -1119,9 +1170,63 @@ int fork_app(char * cmd) /* {{{ */
       break;
 
     case 0:
+      ptr = 0;
+      argc = 0;
+      mode = SKIP;
+      quote_char = '"';
+      memset(args, 0, sizeof(args));
+      cmd_cpy = strdup(cmd);
+      cmd_len = strlen(cmd_cpy);
+
+      /* generate an array of cmd_cpy string */
+      while (ptr < cmd_len) {
+	switch (mode) {
+	  /* skip leading whitespaces */
+	  case SKIP:
+	    if (cmd_cpy[ptr] != ' ' && cmd_cpy[ptr] != '\t') {
+	      if (cmd_cpy[ptr] == '"' || cmd_cpy[ptr] == '\'') {
+		mode = IN_QUOTED;
+		quote_char = cmd_cpy[ptr];
+		ptr++;
+	      }
+	      else {
+		mode = IN_UNQUOTED;
+	      }
+	      args[argc] = &cmd_cpy[ptr];
+	      argc++;
+	    }
+	    ptr++;
+	    break;
+
+	  /* within a quoted string */
+	  case IN_QUOTED:
+	    if (cmd_cpy[ptr] == quote_char) {
+	      cmd_cpy[ptr] = '\0';
+	      mode = SKIP;
+	    }
+	    ptr++;
+	    break;
+
+	  /* within a unquoted string */
+	  case IN_UNQUOTED:
+	    if (cmd_cpy[ptr] == ' ' || cmd_cpy[ptr] == '\t') { /* this is the end of the string */
+	      cmd_cpy[ptr] = '\0';
+	      mode = SKIP;
+	    }
+	    ptr++;
+	    break;
+	}
+      }
+
       setsid(); /* children should not be killed if tpb ends */
-      /* execv("/bin/sh", args); */
-      system(cmd);
+      execv(args[0], args);
+
+      /* shold never be reached, execv only returns on error */
+      fprintf(stderr, _("Unable to fork application \"%s\".\n"), args[0]); 
+      fprintf(stderr, _("HINT:\n")); 
+      fprintf(stderr, _("With tpb version 0.6.2 or later you need to specify applications\n")); 
+      fprintf(stderr, _("using the whole path to the executable program.\n")); 
+      free(cmd_cpy);
       _exit(0);
       return 127;
       break;
@@ -1299,6 +1404,7 @@ int apmiser_running(void) /* {{{ */
   return 0;
 } /* }}} */
 
+#ifdef HAVE_LIBX11
 /* handler for X errors */
 int *xerrorhandler(Display *display, XErrorEvent *event) /* {{{ */
 {
@@ -1312,5 +1418,14 @@ int *xerrorhandler(Display *display, XErrorEvent *event) /* {{{ */
 
   return(NULL);
 } /* }}} */
+#endif /* HAVE_LIBX11 */
+
+/* handler for SIG_CHLD */
+void sig_chld_handler(int signo) /* {{{ */
+{   
+  int status;
+  waitpid(-1, &status, WNOHANG);
+  return;
+} /* }}} */
 
 /* vim600:set fen:set fdm=marker:set fdl=0: */
